--- display.c.orig	2005-11-09 01:16:47.000000000 +0900
+++ display.c	2011-12-31 00:23:52.000000000 +0900
@@ -79,6 +79,29 @@
   return TRUE;
 }
 
+int to_center()
+{
+  int delta, savecursor = cursor;
+  int center_line = (page / lineLength) / 2;
+  int cursor_line = cursor / lineLength;
+
+  if (mode == bySector)
+    return TRUE;
+
+  delta = (cursor_line - center_line) * lineLength;
+
+  if (base + delta < 0)
+    delta = -(base / lineLength) * lineLength;
+
+  while (tryloc(base + delta + page - lineLength) == FALSE)
+    delta -= lineLength;
+
+  move_base(delta);
+  move_cursor(-delta + (savecursor - cursor));
+
+  return TRUE;
+}
+
 
 int computeLineSize(void) { return computeCursorXPos(lineLength - 1, 0) + 1; }
 int computeCursorXCurrentPos(void) { return computeCursorXPos(cursor, hexOrAscii); }
@@ -175,6 +198,12 @@
   printw("-%c%c  %s       --0x%llX", i, i, baseName, base + cursor);
   if (MAX(fileSize, lastEditedLoc)) printw("/0x%llX", getfilesize());
   if (mode == bySector) printw("--sector %d", (base + cursor) / SECTOR_SIZE);
+  {
+    int c = (cursor < nbBytes) ? buffer[cursor] : ' ';
+    printw("--%3d--", c);
+    for (i = 0; i < 8; i++)
+      printw("%c", (c & (1 << (7 - i))) ? '1' : '0');
+  }
 
   move(cursor / lineLength, computeCursorXCurrentPos());
 }
@@ -182,10 +211,19 @@
 void displayLine(int offset, int max)
 {
   int i;
+  int vc, cursor_line, cursor_clmn, attr, cursor_attr;
 
-  PRINTW(("%08lX   ", (int) (base + offset)));
+  cursor_line = (cursor >= offset && cursor < offset + lineLength);
+  PRINTW(("%08lX%c", (int) (base + offset), (cursor_line ? '>' : ' ')));
+  if (cursor < max) {
+    cursor_attr = bufferAttr[cursor];
+    bufferAttr[cursor] = cursor_attr | A_UNDERLINE;
+  }
   for (i = offset; i < offset + lineLength; i++) {
-    if (i > offset) MAXATTRPRINTW(bufferAttr[i] & MARKED, (((i - offset) % blocSize) ? " " : "  "));
+    cursor_clmn = ((i % lineLength) == (cursor % lineLength));
+    attr = (cursor_line && hexOrAscii) ? A_UNDERLINE : 0;
+    vc = (cursor_clmn && (hexOrAscii || (!hexOrAscii && cursor_line))) ? '|' : ' ';
+    MAXATTRPRINTW((bufferAttr[i] & MARKED) | attr, (((i - offset) % blocSize) ? "%c" : " %c", vc));
     if (i < max) {
       ATTRPRINTW(
 #ifdef HAVE_COLORS
@@ -194,16 +232,23 @@
 		  buffer[i] < ' ' ? COLOR_PAIR(2) : 
 		  buffer[i] >= 127 ? COLOR_PAIR(3) : 0) |
 #endif
+		 attr |
 		 bufferAttr[i], ("%02X", buffer[i]));
     }
     else PRINTW(("  "));
   }
   PRINTW(("  "));
   for (i = offset; i < offset + lineLength; i++) {
+    cursor_clmn = ((i % lineLength) == (cursor % lineLength));
+    attr = (!hexOrAscii && (cursor_clmn || cursor_line)) ? A_REVERSE : 0;
     if (i >= max) PRINTW((" "));
-    else if (buffer[i] >= ' ' && buffer[i] < 127) ATTRPRINTW(bufferAttr[i], ("%c", buffer[i]));
-    else ATTRPRINTW(bufferAttr[i], ("."));
+    else if (buffer[i] >= ' ' && buffer[i] < 127) ATTRPRINTW(bufferAttr[i] | attr, ("%c", buffer[i]));
+    else ATTRPRINTW(bufferAttr[i] | attr, ("."));
+  }
+  if (cursor < max) {
+    bufferAttr[cursor] = cursor_attr;
   }
+  PRINTW((""));
 }
 
 void clr_line(int line) { move(line, 0); clrtoeol(); }
